// Music 256a / CS 476a | fall 2016
// CCRMA, Stanford University
//
// Author: Romain Michon (rmichonATccrmaDOTstanfordDOTedu)
// Description: Simple synth demonstrating how to use the STK with JUCE.

#ifndef MAINCOMPONENT_H_INCLUDED
#define MAINCOMPONENT_H_INCLUDED

#include "../JuceLibraryCode/JuceHeader.h"
#include "stk/SineWave.h" // STK sine osc
#include "stk/BlitSaw.h" // STK sawtooth osc

using namespace stk; // required for the STK to work

class MainContentComponent :
public AudioAppComponent,
private Slider::Listener,
private ToggleButton::Listener
{
public:
    MainContentComponent()
    :   currentSampleRate (0.0),
    gain(0.0),
    sineOnOff(0.0),
    sawOnOff(0.0)
    {
        addAndMakeVisible (frequencySlider);
        frequencySlider.setRange (50.0, 5000.0);
        frequencySlider.setSkewFactorFromMidPoint (500.0);
        frequencySlider.setValue(1000);
        frequencySlider.addListener (this);
        
        addAndMakeVisible(frequencyLabel);
        frequencyLabel.setText ("Frequency", dontSendNotification);
        frequencyLabel.attachToComponent (&frequencySlider, true);
        
        addAndMakeVisible (gainSlider);
        gainSlider.setRange (0.0, 1.0);
        gainSlider.setValue(0.5);
        gainSlider.addListener (this);
        
        addAndMakeVisible(gainLabel);
        gainLabel.setText ("Gain", dontSendNotification);
        gainLabel.attachToComponent (&gainSlider, true);
        
        addAndMakeVisible(sineButton);
        sineButton.addListener(this);
        
        addAndMakeVisible(sawButton);
        sawButton.addListener(this);
        
        addAndMakeVisible(sineLabel);
        sineLabel.setText ("Sine", dontSendNotification);
        sineLabel.attachToComponent (&sineButton, true);
        
        addAndMakeVisible(sawLabel);
        sawLabel.setText ("Saw", dontSendNotification);
        sawLabel.attachToComponent (&sawButton, true);
        
        setSize (600, 130);
        setAudioChannels (0, 1); // no inputs, one output
    }
    
    ~MainContentComponent()
    {
        shutdownAudio();
    }
    
    void resized() override
    {
        const int labelColumnWidth = 80;
        const int componentsHeight = 20;
        frequencySlider.setBounds (labelColumnWidth, 10, getWidth() - labelColumnWidth - 20, componentsHeight);
        gainSlider.setBounds (labelColumnWidth, 40, getWidth() - labelColumnWidth - 20, componentsHeight);
        sineButton.setBounds (labelColumnWidth, 70, getWidth() - labelColumnWidth - 20, componentsHeight);
        sawButton.setBounds (labelColumnWidth, 100, getWidth() - labelColumnWidth - 20, componentsHeight);
    }
    
    void sliderValueChanged (Slider* slider) override
    {
        if (slider == &frequencySlider)
        {
            if (currentSampleRate > 0.0){
                sine.setFrequency(frequencySlider.getValue());
                saw.setFrequency(frequencySlider.getValue());
            }
        }
        else if (slider == &gainSlider)
        {
            if (currentSampleRate > 0.0){
                gain = gainSlider.getValue();
            }
        }
    }
    
    void buttonClicked (Button* button) override
    {
        if(button == &sineButton && sineButton.getToggleState()){
            sineOnOff = 1.0;
        }
        else{
            sineOnOff = 0.0;
        }
        if(button == &sawButton && sawButton.getToggleState()){
            sawOnOff = 1.0;
        }
        else{
            sawOnOff = 0.0;
        }
    }
    
    void prepareToPlay (int /*samplesPerBlockExpected*/, double sampleRate) override
    {
        currentSampleRate = sampleRate;
        Stk::setSampleRate(sampleRate); // STK must know the sampling rate
        
        // setting default values for STK parameters
        sine.setFrequency(1000.0);
        saw.setFrequency(1000.0);
    }
    
    void releaseResources() override
    {
    }
    
    void getNextAudioBlock (const AudioSourceChannelInfo& bufferToFill) override
    {
        float* const buffer = bufferToFill.buffer->getWritePointer (0, bufferToFill.startSample);
        
        for (int sample = 0; sample < bufferToFill.numSamples; ++sample)
        {
            float sineSample;
            float sawSample;
            if(sineOnOff){
                sineSample = sine.tick()*gain; // computing next sine wave sample
            }
            else{
                sineSample = 0.0;
            }
            if(sawOnOff){
                sawSample = saw.tick()*gain; // computing next sawtooth sample
            }
            else{
                sawSample = 0.0;
            }
            buffer[sample] = sineSample+sawSample; // filling output buffer
        }
    }
    
    
private:
    Slider frequencySlider;
    Slider gainSlider;
    ToggleButton sineButton;
    ToggleButton sawButton;
    
    Label frequencyLabel;
    Label gainLabel;
    Label sineLabel;
    Label sawLabel;
    
    SineWave sine; // sine osc
    BlitSaw saw; // sawtooth osc
    
    double currentSampleRate, gain, sineOnOff, sawOnOff; // [1]
    
    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MainContentComponent)
};

Component* createMainContentComponent()     { return new MainContentComponent(); }


#endif  // MAINCOMPONENT_H_INCLUDED
